dnl ----------------------=|=--- head.m4 ---=|=-----------------------------
dnl --=-- Initialization of the TUNES i386 BIOS API --=--
dnl
dnl
dnl Lots of info taken from the sources of the Linux console driver.
dnl
Provide({head},{
dnl
dnl


rminit

__codeseg__({
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;; API RM Entry Point ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

entry API_Entry_Point


API_Entry_Point:
;;; Called by the boot loader.
;;Input:
;;; the whole API must be loaded at linear address APIOFS (0x00001000)
;; cs,ds,es,ss must be zero
;; [System_Length]: the total length of the system file loaded at APIOFS
;; [COMMAND_LINE]: a zero-terminated ascii string (command line arguments)
;;			of max length 2047 not to overwrite the API
;; sp=COMMAND_LINE-14  (!)Beware, stack is less than 768 bytes deep
;; dword [sp+10]: Total Extended Memory
;; dword [sp+6]: Total Conventional Memory
;; dword [sp+2]: Available Conventional Memory
;; word [sp]: CPU type
;;Output:
;; Never exits,
;; but launches TUNES instead.


pushdef({codeaddr},{$1})	; until we moved the RM part, code and strings
pushdef({stringaddr},{$1})	; are at their original address

;;; Init Console
   ;; Define Video Mode: text mode 80x25, 16 colors, clear screen @ 0xB8000
	mov ax,#0x0003
	int #0x10

   ;; Print Init Message
	__("Initializing TUNES:",CRLF)

   ;; Set the keyboard repeat rate to the max
	mov ax,#0x0305
	clr(bx)
	int #0x16


;;; Get parameters from the LOADER
   ;; Get CPU type
	pop word [CPUType]
	zdwoz({"CPU:       0x"},[CPUType],{CRLF})

   ;; Get available conventional memory
	pop eax			; optimize for below
	mov [AvConvMem],eax
	zdwoz({"AvConvMem: 0x"},[AvConvMem],{CRLF})

   ;; Get total conventional memory
	pop dword [ConvMem]
	zdwoz({"ConvMem:   0x"},[ConvMem],{CRLF})

   ;; Get available/total extended memory
	pop dword [XtdMem]
	zdwoz({"XtdMem:    0x"},[XtdMem],{CRLF})

   ;; Get command line
   ;; (nothing to do: loader put it at COMMAND_LINE, zero-ended)
	zAsciiZz({"Command line is:",CRLF},#COMMAND_LINE,{CRLF})


;;; Initialize some PM Part variables
   ;; save real-mode IDT
	sidt [RM_IDT_address]
	zWZonez({"IDT: "},ds,RM_IDT_address,3,{CRLF})

   ;; Putting the RM Part seg everywhere has been done below !

   ;; ain't that all ?
	;.....


;;; Move the Real-Mode Part to where it belongs

   ;; Compute the target location for RM Part.
	;mov eax,[AvConvMem]	; (!) optimized above
	;zdwoz({"AvConvMem: 0x"},[AvConvMem],{CRLF}) ; let's check it
	shr eax,#4
	sub ax,#0x1000		; 64 KB in para

   ;; Firstly, Initialize some variables with it
	mov [RM_Part_seg],ax	; tell the PM part where to find the RM part !
	mov [RM_Target_seg],ax			; target when back into RM...
	mov [Actual_INT_Interceptor_seg],ax	; RM->PM interrupt interceptor
	zwoz({"RM_Part_seg: "},ax,{CRLF})

   ;; Then, Do move
	__("Moving the Real-Mode Part to where it belongs... ")
	mov si,#rmpart_actbeg
	mov es,ax
	mov di,#rmaddr(rmpart_actbeg)
	mov cx,#(rmpart_actlen+1)/2
	zaddrz({CRLF,"Source: "},ds,si)
	zaddrz({" Dest: "},es,di)
	zwoz({" Count: 0x"},cx,{CRLF})
	cld
	rep
	movsw

   ;; Clear the zone containing uninitialized RM part data
   ;; So we can be confident that data defined up there is properly zeroed
	__("Clearing the RM BSS... ")
	clr(ax)
	mov cx,#(rmpart_bsslen+1)/2
	zaddrz({CRLF,"Dest: "},es,di)
	zwoz({" Count: 0x"},cx,{CRLF})
	;zaddrz({CRLF,"cs:ip="},cs,ip,{CRLF})
	rep
	stosw

	zzzz({
	  ppmov(fs,0xB800)
	  seg fs
	  mov [0],dword #0x07450744
	  pushing({ds,es},{
	    ppmov(ds,#0)
	    ppmov(es,ds)
	    __("Done.",CRLF)
	  seg fs
	  mov [4],dword #0x07550742
	})})

;;; Jump up in (initialization portion of) moved RM Part of the API ...
   ;; use stack and data in rmpart
	ppmov(ds,es)
	ppmov(ss,ds)
	mov esp,#rmaddr(RMPartStackTop)
	ppmov(es,#0)
	zzzz({
	  seg fs
	  mov [8],dword #0x07200747
	})
     popdef({stringaddr})

   ;; jump in RM Part...
	STOP(fs,0,6,00,'M,0x07)
	__("Jumping up... ")
	STOP(fs,0,7,00,'E,0x07)
	pushm(#rmaddr(UpThere),ds)
	retf

     popdef({codeaddr})
})


rminit

__codeseg__({

UpThere:
	STOP(es,0,9,00,'P,0x07)
;;; Initialization routine in moved RM part
;;Input:
;;; The RM part has been moved up
;;; offset of data is rmaddr(original_offset).
;; cs,ds,ss,es=RMPART,RMPART,RMPART,0
;; esp=rmaddr(RMPartStackTop)
;;Output:
;;; Now, we shall go in protected mode as soon as possible:
;;; everything is *so* much easier from there,
;;; and we write all support routines for PM anyway,
;;; so why not use them or write crippled redundant real-mode counterparts ?

	__("Here I am up in RM part !",CRLF)


;;; Initialize RM Part variables
   ;; compute RM and PM values for cr0
	mov eax,cr0
	mov [rmaddr(RM_cr0)],eax
	and eax,#~(CR0_PG|CR0_CD|CR0_NW|CR0_AM|CR0_WP)
	or al,#CR0_PE
	mov [rmaddr(PM_cr0)],eax

;;; Disable all interrupts
	cli
	__({"Disabling IRQs"},CRLF)
	call RM_Mask_IRQs
	__({"Disabling NMI"},CRLF)
	call RM_Disable_NMI
	__({"All interrupts disabled"},CRLF)

;;; Jump into the Protected Mode
	__("Let's jump into protected mode !",CRLF)
	br GoProtect



;;; Here, hack,hack,hack: a frame for Return_From_R86 (see pmpart.m4)
;  Well, after all, because and the space wasted by unneeded registers
; in the frame, and a special case added to the server, paging enabler,
; or R86 returner this will be much longer than it is. And to be efficient,
; we wouldn't even remove the code-modifying we're doing.
; We wouldn't gain anything at wasting time doing that. Let's not.

})


pminit

__codeseg__({
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;; API PM Entry Point ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

define({PrGDT},
{
	SHOWL("GDTlim0 GDTbase GDT...")
	sub esp,#17*2*4-2
	sgdt [esp]
	push word #0
	mov esi,[esp+4]
	add esi,ebp
	ppmov(ecx,17*2-2)
	lea edi,[esp+8]
	rep
	movsd
	mov esi,esp
	call ShowDZone
	call ShowDZone
})


PM_Entry_Point:
;;; Initialize Registers for protected mode
;;Input:
;; we just entered protected mode;
;; the pmpart is in its original place at segment 0
;; [XtdMem] contains the amount of available extended memory, at least 128KB.
;;Output:
;; cs,ds,es,fs,ss now allow addressing of the whole linear (physical) memory
;; ebp points to the target address for the top of the PM part
;; esp points to the target address for __Stack_Top__
;; some variables are initialized: [Id_Pgtbl_Num]
;; eflags is cleared (particularly no interrupts & string instructions upward)
;; except for IOPL which is set to 3.
;; other gregisters are still uninitialized
   ;; Initialize ds,es,fs
	ppmov(ds,#ALLMEMDS)
	ppmov(es,ds)
	ppmov(fs,ds)
	STOP(ds,0xB8000,78,00,'P,0x49)
	STOP(es,0xB8000,79,00,'M,0x49)

   ;; compute the target location for PM Part.
	; (?) should we limit it to 16 MB, as linux allows to do ???
	; well, in that case, it'd be better to change the LOADER, I guess.
	; people would also add a handler for extra memory as buffer...
      ;; EBP is top of memory
	mov ebp,[XtdMem]
	add ebp,#0x100000 ; Xtdmem is above 1 MB...
      ;; EAX number of 4MB page tables needed to map the memory
	mov eax,ebp
	dec eax
	sar eax,22
	inc eax
	mov [Id_Pgtbl_Num],eax
	shl eax,12
	;mov [Id_Pgtbl_Sz],eax
      ;; for further paging, we must be 4KB aligned...
	and ebp,#PAGEMASK
      ;; Possibly limit us to 16 MB
	ifdef({LIMIT_TO_16MB},{
		__IF__(a)
		  mov ecx,#0x1000000
		  cmp ebp,ecx
		__THEN__
		  mov ebp,ecx
		__ENDIF__
	})
      ;; let enough space for identity mapping page tables
	sub ebp,eax

   ;; Initialize SS:
   ;; same all memory segment, stack physically where it belongs.
	ppmov(ss,ds)
	lea esp,[ebp+pmaddr(__Stack_Top__)]

	;NB: I lost a lot of time finding a bug:
	;  ***addressing using ebp implicitly uses ss***,
	;  but I had ss still in the rmpart segment,
	;  so anytime I tried to use ebp-addressing
	;  meaning ds not ss, it just crashed (GPF) 8-( )-8


   ;; Initialize EFLAGS.
   ;; Particularly, clear interrupts, allow user to do IO,
   ;; Initialize convention of direction flag being clear (moving upward),
   ;; Clear some nasty flags that real-mode may have given us: NT, AC, etc.
	ppmov(fd,#FLAG_IOPL3)


;;; Print "Welcome to "
;;Input:
;; we have a color adaptor in text mode with buffer at 0xB8000
;;Ouput:
;;; message printed
;; ebp not modified
	mov edi,#0xB8000+2*(00+0*80)
	mov eax,#0x07650757	; 'We'
	stosd
	mov eax,#0x0763076C	; 'lc'
	stosd
	mov eax,#0x076D076F	; 'om'
	stosd
	mov eax,#0x07200765	; 'e '
	stosd
	mov eax,#0x076F0774	; 'to'
	stosd


;;; Move the Protected-Mode Part to where it belongs: up extended memory.
;;Input:
;; ebp contains target address for the PM part.
;;Output:
;; ebp unchanged
;; the PM part is moved, and the bss is cleared.
   ;; Do Move
	mov esi,#pmpart_actbeg
	lea edi,[ebp+pmaddr(pmpart_actbeg)]
	mov ecx,#(pmpart_actlen+3)/4
	rep
	movsd

   ;; Clear the zone containing uninitialized PM part data
   ;; So we can be confident that data defined up there is properly zeroed
   ;; Could someone tell me what "bss" stands for ???
	clr(eax)
	mov ecx,#(pmpart_bsslen+3)/4
	rep
	stosd


;;; Initialize the GDT
;;Input:
;; ebp contains the physical address of the PM part.
;;Output:
;; The PMPART and RMPART entries of the GDT are initialized.
;; The [PM_GDT_address] variable in RM part is initialized
;; The GDT register is loaded.
;; ebp unchanged
;; ebx contains the physical address of the RMPART
;;Note:
;;   Lookout: KL0 entries are still at their ALLMEM values;
;; but this shouldn't matter if we call Init_Paging before to use them
;; (in particular, before any interrupt occurs).

   ;; Copy the original GDT
   ;; No more necessary since the original is now the final one, too !
	;mov esi,#Orig_GDT
	;lea edi,[ebp+pmaddr(__GDT__)]
	;mov ecx,#Orig_GDT_Len/4
	;rep
	;movsd

   ;; Modify the PMPART entries to point up in physical memory
	mov ax,#0xC09A
	mov ebx,ebp
	mov ecx,#0xFFFFF
	lea edx,[ebp+pmaddr(__GDT__+PMPARTCS)]
	call SetDescriptor
	add edx,#PMPARTDS-PMPARTCS
	mov al,#0x92
	call SetDescriptor

   ;; Modify the RMPART entries
	mov ebx,[RM_Part_seg]
	shl ebx,4
	mov ah,#0x10
	shr ecx,4
	add edx,#RMPARTDS-PMPARTDS
	call SetDescriptor
	mov al,#0x9A
	add edx,#RMPARTCS-RMPARTDS
	call SetDescriptor

   ;; Set the GDT variable in RM part, so further RM->PM switching works
	lea eax,[edx-RMPARTCS]
	mov [ebx+rmaddr(PM_GDT_base)],eax
	mov word [ebx+rmaddr(PM_GDT_lim)],#GDT_SIZE-1

   ;; Re-load the GDT
	lgdt [ebx+rmaddr(PM_GDT_address)]



;;; Re-Initialize registers
;;Input:
;; cs,ds,es,ss,fs point to ALLMEM
;; ebp contains the physical address of the PM part
;; ebx contains the physical address of the RM part
;;Output:
;;; We now execute code at its definitive address
;; cs,ds,es,ss,gs point to the PMPART (*not* KL0)
;; esp is adjusted stack is at the same physical place as before.
;; ebp is unchanged
;; eax points to the logical address of physical memory in PMPART
;; fs still points to ALLMEM
;; Some variables are initialized or adjusted:
;; [PM_Part_Phys], [RM_Part_Phys], [PM_IDT_addr] (GDT done earlier)
;; [RM_Part_Addr], [PhysicalMem], [Video_mem_base]
;; [V86_REQUEST_Adjust-4]
;; [PM_Target_ofs]
;; [X86_FLAGS_Addr], [GDT_Addr_Addr]

   ;; Adjust cs:eip
	STOP(ds,0xB8000,78,01,'U,0x78)
	jmpi pmaddr(UpTherePM),PMPARTCS
   UpTherePM:
	STOP(ds,0xB8000,79,01,'P,0x78)

   ;; Ajust ds,es,gs
	ppmov(ds,#PMPARTDS)
	ppmov(es,ds)
	ppmov(gs,ds)

   ;; Adjust ss,esp
	ppmov(ss,ds)
	sub esp,ebp

   ;; Initialize RM-related variables
	mov eax,ebx
	mov [pmaddr(RM_Part_Phys)],eax
	add [pmaddr(V86_REQUEST_Adjust-4)],eax
	sub eax,ebp
	mov [pmaddr(RM_Part_Addr)],eax
	add [pmaddr(X86_FLAGS_Addr)],eax
	add [pmaddr(GDT_Addr_Addr)],eax
	mov [eax+rmaddr(PM_Target_ofs)],#pmaddr(Return_From_R86)

   ;; Initialize PM-related variables
	mov eax,ebp
	mov [pmaddr(PM_Part_Phys)],eax
	add [pmaddr(PM_IDT_base)],eax
	neg eax
	mov [pmaddr(PhysicalMem)],eax
	add [pmaddr(Video_mem_base)],eax


;;; Proudly, hackishly, print " Tunes !", and clear rest of screen
;;Input:
;; eax points to the physical memory in PMPART
;;Output:
;; eax-20 now points to video frame buffer in PMPART
;; ebp unchanged
;; hack score incremented ;-)
   ;; We're assuming no interrupts, color screen at B800, etc.
	xchg eax,esp
	add esp,#0xB8000+2*25*80
	mov ecx,#(80*25-18)/2
	Loopno0(push #0x07200720)
	pushm(#0x0F540F20, #0x0F6E0F75, #0x0F730F65, #0x07210720)
	xchg eax,esp


;;; Setup Paging
;;Output:
;; Paging enabled, with segments now KL0 executing up in linear memory. Wow !
;; gdt and idt registers re-loaded for paging, too
;; only eax,esi are wasted
	STOP(fs,0xB8000,25,0,'P,0x29)
	DLINE(1)
zzzz({
	movm(
	 {ebp,[pmaddr(PhysicalMem)]},
	 {eax,[pmaddr(GDT_Addr_Addr)]},
	 {edx,[pmaddr(RM_Part_Addr)]},
	 {cx,[eax]},
	 {ebx,[eax+2]})
	STOP(fs,0xB8000,9,0,'X,0x2F)
	z__z
	STOP(fs,0xB8000,9,0,'Y,0x2F)
	PrGDT
	STOP(fs,0xB8000,9,0,'Z,0x2F)
})
	call Init_Paging
	STOP(fs,0xB8000,26,0,'G,0x29)
	STOP(es,0xB8000,27,0,'E,0x29)
	STOP(ds, ebx-20,28,0,'N,0x29)
zzzz({
	SHOWL("Paging Enabled")
	movm(
	 {ebp,0},
	 {eax,[pmaddr(GDT_Addr_Addr)]},
	 {edx,[pmaddr(RM_Part_Addr)]},
	 {cx,[eax]},
	 {ebx,[eax+2]})
	STOP(fs,0xB8000,9,0,'X,0x2F)
	z__z
	STOP(fs,0xB8000,9,0,'Y,0x2F)
	PrGDT
	STOP(fs,0xB8000,9,0,'Z,0x2F)
})


;;; Setup Initial Task register and structures
	DMSG({TR TSSs Switch},50,24,0x09,fs,0xB8000)
	movzil(a,#DummyTSS)
	ltr ax
	LETTER(2)
	movm(	{edi,#pmaddr(K_TSS)},
		{esi,esp},
		{al,#KL0CS},
		{ebx,#pmaddr(SwitchMe)},
		{ecx,#FLAG_IOPL3},
		{[pmaddr(K_TSS)+TSS32_EAX],#0xDEADBEEF})
	call Init_TSS
	LETTER(4)
	;z__z
	LETTER(2)
	jmpi 0,KTSS
SwitchMe:
	LETTER(3)
	;z__z
	LETTER(3)
;; With flag NT set, iretd would result in an exception 0xA,
;;because Dummy_TSS is invalid. Why it is invalid I don't know.


;;; Setup Interrupts
;;Input:
;; idt register previously loaded
;;Output:
;; IDT zone initialized to manage exceptions reflect IRQs into RM.
;; PIC chips reprogrammed.
;; ebp,ebx unmodified
	STOP(fs,0xB8000,0,24,'I,0x0B)
   ;; Redirect Hardware Interrupts
	call Setup_PM_8259
	STOP(fs,0xB8000,1,24,'H,0x03)
   ;; Initialize PM IDT:
      ;; Move the parts of original IDT: exception handlers
	mov esi,#pmaddr(Orig_IDT_exc)
	mov edi,#pmaddr(__IDT__)
	mov ecx,#Orig_IDT_exc_Len/4
	rep
	movsd
	STOP(fs,0xB8000,4,24,'e,0x03)
      ;; Move the parts of original IDT: interrupt handlers
	add edi,#IRQ_INT_BASE*8-Orig_IDT_exc_Len
	mov ecx,#Orig_IDT_irq_Len/4
	rep
	movsd
	STOP(fs,0xB8000,5,24,'i,0x03)
   ;; Do NOT load IDT register: it is done in before according to PM/RM
	;lidt [pmaddr(PM_IDT_address)]	; necessary if not paging.
	;STOP(fs,0xB8000,6,24,'l,0x03)

;;; Setup the PM->RM reflector
;;Output:
;; ebp conserved
   ;; Initialize RM IDT:
	call Init_RM_IDT
	STOP(fs,0xB8000,2,24,'R,0x03)
   ;; Initialize ReflINT V86INT:
	mov eax,[pmaddr(RM_Part_seg)]
	mov [pmaddr(ReflINT_V86INT+V86INT_SS)],eax
	mov [pmaddr(ReflINT_V86INT+V86INT_ESP)],#rmaddr(RMPartStackTop)
	mov [pmaddr(ReflINT_V86INT+V86INT_CS)],eax
	mov [pmaddr(ReflINT_V86INT+V86INT_EIP)],#rmaddr(RM_Idle)
	;(!) EFLAGS must be zeroed. ok because defined as "uninitialized" data
	STOP(fs,0xB8000,3,24,'V,0x03)
   ;; Initialize the Current RM V86INT
	;done at compile time (see definition for Current_RM_V86INT)


;;; Setup RM->PM reflector
;;Output:
;; RM interceptor IDT initialized.
;; ebp conserved
   ;; Initialize RM Interceptor IDT
	; if in "intercept all RM interrupts" mode,
	; all interrupts point to the same place
	; except that the low byte of cs indicates the interrupt number.
	mov ebx,[pmaddr(RM_Part_Addr)]
	;mov eax,[pmaddr(RM_Part_seg)] ; (!) must have been done just before
	add eax,eax
	lea eax,[eax*8+rmaddr(INT_Interceptor)-0x2000] ;8*2=16 !!!
	mov edx,eax
	and edx,#0x00FFF
	and eax,#0xFF000
	shl eax,#12
	lea eax,[eax+edx+0x2000]
	mov edi,#pmaddr(__RM_IDT__)
	mov cx,#256
	Loopno0({
	  stosd
	  add eax,#0x10000-0x10
	})
	STOP(fs,0xB8000,6,24,'R,0x03)


;;; Ready interrupts
;;Output:
;; the system is ready to process interrupts, but they are still disabled.
;; eax lost
	call Enable_NMI
	call Unmask_IRQs
	call Flush_Interrupts
	STOP(fs,0xB8000,70,24,'I,0x0C)




;;;;;;; (!!!) Below this line, code is under heavy development/testing ;;;;;;;

DMSG({So now, we're testing at last; don't try to efhzoe me baby.},
	0,22,0x0E,fs,0xB8000))
	LETTER(10)


	OPTDIE(X86)

;;; Initialize Test_V86INT
	DBG(V)
	mov ebx,#pmaddr(Test_V86INT)
	mov eax,[pmaddr(RM_Part_seg)]
	mov [ebx+V86INT_CS],eax
	mov [ebx+V86INT_DS],eax
	mov [ebx+V86INT_SS],eax
	DBG(86)

;	sti
;;;; see archive of routines in test.S



;;;;;;;;;;;;;;;;;;;;;;;;; Now let's go to the LLL ;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Initialize a stream input to what follows the API in the image file...
	mov ebp,[pmaddr(PM_Part_Phys)]
	mov ecx,#APISIZE
	mov eax,[ebp+System_Length]
	sub eax,ecx
	add ecx,ebp
	mov [pmaddr(Init_Stream_base)],ecx
	mov [pmaddr(Init_Stream_len)],eax
	;......


;;; Here we are (Pphew !)
;;; Jump into LLL code
	STOP(fs,0xB8000,71,24,'J,0x0C)
	;......
	STOP(fs,0xB8000,72,24,'K,0x0C)


;;;;;;;;;;;;;;; When TUNES runs, there will be no return ;;;;;;;;;;;;;;;;;;;;
; ( but TUNES being able to run arbitrary continuations,
; it will be possible to halt, reboot, or launch another OS ;-)


;;; Exit back to real mode
   ;; Bye bye !

	STOP(fs,0xB8000,78,24,'B,0x0C)

	cli
	call Mask_IRQs
	call Disable_NMI
	call Setup_RM_8259
	call Flush_Interrupts
	call DumpV86INT
	mov ecx,50
	call sleep
	call Call_R86

   ;; Error ?
	STOP(fs,0xB8000,79,24,'E,0x8F)
	___die___ ; This point should never be reached !


Test_V86INT:
	.long 1,2,3,4,5,6,7,8		; PUSHAD_
	.long 0,0			; EIP,CS
	.long 0x00003046		; EFLAGS
	.long rmaddr(rmpart_actbeg),0	; ESP,SS
	.long 0xB800,0,0,0		; ES,DS,FS,GS

})


__dataseg__({
;;;;;;;;;;;;;;;;;;;;;;;;;; Initialization Data ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

Orig_IDT_exc:
irp({exc_handler},
{Division_Error_Trap,
 Single_Step_Trap,
 NMI_Trap,
 BreakPoint_Trap,
 Overflow_Trap,
 Out_of_Bound_Trap,
 Invalid_Opcode_Trap,
 Emulate_Coproc_Trap,
 Double_Fault_Stop,
 Coproc_Segment_Stop,
 Invalid_TSS_Fault,
 Segment_Not_Present_Fault,
 Stack_Fault,
 General_Protection_Fault,
 Page_Fault,
 Reserved_0F_Fault,
 Coprocessor_Error_Fault,
 Alignment_Check_Fault},
{DefGate(KL0CS,pmaddr(exc_handler),	0,0x8E)
})
Orig_IDT_exc_end:
Orig_IDT_exc_Len = Orig_IDT_exc_end - Orig_IDT_exc

;Orig_IDT_serv:
;Orig_IDT_serv_end:
;Orig_IDT_serv_Len = Orig_IDT_serv_end - Orig_IDT_serv

Orig_IDT_irq:
  DefGate(KL0CS,pmaddr(IRQ0_Timer),	0,0x8E)	; 0
  DefGate(KL0CS,pmaddr(IRQ1_Keyboard),	0,0x8E)	; 1
  DefGate(KL0CS,pmaddr(IRQ2_Cascade),	0,0x8E)	; 2
  DefGate(KL0CS,pmaddr(IRQ3),		0,0x8E) ; 3
  DefGate(KL0CS,pmaddr(IRQ4),		0,0x8E) ; 4
  DefGate(KL0CS,pmaddr(IRQ5),		0,0x8E) ; 5
  DefGate(KL0CS,pmaddr(IRQ6),		0,0x8E) ; 6
  DefGate(KL0CS,pmaddr(IRQ7),		0,0x8E) ; 7
  DefGate(KL0CS,pmaddr(IRQ8_RTC),	0,0x8E) ; 8
  DefGate(KL0CS,pmaddr(IRQ9),		0,0x8E) ; 9
  DefGate(KL0CS,pmaddr(IRQ10),		0,0x8E) ; A
  DefGate(KL0CS,pmaddr(IRQ11),		0,0x8E) ; B
  DefGate(KL0CS,pmaddr(IRQ12),		0,0x8E) ; C
  DefGate(KL0CS,pmaddr(IRQ13),		0,0x8E) ; D
  DefGate(KL0CS,pmaddr(IRQ14),		0,0x8E) ; E
  DefGate(KL0CS,pmaddr(IRQ15),		0,0x8E) ; F
Orig_IDT_irq_end:
Orig_IDT_irq_Len = Orig_IDT_irq_end - Orig_IDT_irq

})


__pmdataend__({
;;; (!) This takes place at the very end of pmdata segment

.align 16
Orig_GDT:
__GDT__:
 DefDescriptor(0,0x00000,0x00,0x00)				; NULL
 DefDescriptor(0,0xFFFFF,0xF2,0xC0)	; (!) PL3 ALLMEM access	; TEMPSEG
 DefDescriptor(0,0xFFFFF,0x9A,0xC0)				; ALLMEMCS
 DefDescriptor(0,0xFFFFF,0x92,0xC0)				; ALLMEMDS
 DefDescriptor(rmpart_end-0x10000,0xFFFF,0x9A,0x00)		; RMPARTCS
 DefDescriptor(rmpart_end-0x10000,0xFFFF,0x92,0x00)		; RMPARTDS
 DefDescriptor(pmpart_end,0xFFFFF,0x9A,0xC0)			; PMPARTCS
 DefDescriptor(pmpart_end,0xFFFFF,0x92,0xC0)			; PMPARTDS
 DefDescriptor(0,0xFFFFF,0x9A,0xC0)				; KL0CS
 DefDescriptor(0,0xFFFFF,0x92,0xC0)				; KL0DS
 DefDescriptor(0,0xFFFFF,0xBA,0xC0)				; KL1CS
 DefDescriptor(0,0xFFFFF,0xB2,0xC0)				; KL1DS
 DefDescriptor(0,0xFFFFF,0xDA,0xC0)				; KL2CS
 DefDescriptor(0,0xFFFFF,0xD2,0xC0)				; KL2DS
 DefDescriptor(0,0xFFFFF,0xFA,0xC0) ; (!!) normal limit 0xBFFFF	; KL3CS
 DefDescriptor(0,0xFFFFF,0xF2,0xC0) ; (!!) normal limit 0xBFFFF	; KL3DS
 DefGate(KL0CS,pmaddr(KL0_CallGate),0,0x8C)	; ?		; KL0CG
 DefGate(KL0CS,pmaddr(KL1_CallGate),0,0xAC)			; KL1CG
 DefGate(KL0CS,pmaddr(KL2_CallGate),0,0xCC)	; ?		; KL2CG
 DefGate(KL0CS,pmaddr(KL3_CallGate),SysCallArgC,0xEC)		; KL3CG
 DefDescriptor(pmaddr(K_TSS),    0xFFFF,0x89,0x00)		; KTSS
 DefDescriptor(pmaddr(Dummy_TSS),0xFFFF,0x89,0x00)		; DummyTSS
 DefDescriptor(pmaddr(V86_TSS),  0xFFFF,0x89,0x00)		; V86TSS
 DefDescriptor(pmaddr(User_TSS), 0xFFFF,0x89,0x00)		; UserTSS
Orig_GDT_end:
Orig_GDT_Len = Orig_GDT_end-Orig_GDT
;; (!!!) Must be the same as Orig_GDT_SIZE
zzz_CheckMeZero = Orig_GDT_Len-Orig_GDT_SIZE

})


;;;;;;;;;;;;;;;;;;;;;;; Test code for R/P mode switching ;;;;;;;;;;;;;;;;;;;

rminit

__codeseg__({
;;;;

;;;; Test code for R86/V86 reflectors...

Test_V86:
V86_Test:
	cli
	STOP(es,0,40,0,'V,0xA)
	mov [rmaddr(RM_DumpLine)],#2
	rz__z
	STOP(es,0,41,0,'V,0xA)

;	mov ecx,100000000
;BBBB:	dec ecx
;	jnz BBBB

;	mov ax,[0x100000]
;	cli

	STOP(es,0,42,0,'V,0xA)
	PUTS({"Hello, V86 World !"},CRLF)
	;CGETC
	STOP(es,0,43,0,'T,0xA)
	push cs
	call X86_RETURN

R86_Test:
	STOP(es,0,40,24,'R,0xA)
	;pushing(es,{
	;mov ax,#0x0003 ; clear screen
	;int #0x10
	;})
	;cli
	mov [rmaddr(RM_DumpLine)],#15
	rz__z
	CGETC({{"Hello, R86 World !"},CRLF})
	STOP(es,0,41,23,'T,0xA)
	rz__z
	STOP(fs,0xB8000,42,23,'M,0xA)
	push cs
	call X86_RETURN

RM_Return_Point:
; Nothing to restore: every was restored by the R86 handler !
; Show that we got there
	STOP(fs,0xB8000,0,9,'R,0xE)
	STOP(es,0,75,24,'R,0xC)		; 'R' for RealMode
	STOP(es,0,76,24,'e,0xC)		; 'e' for RealMode
; Try accessing ALLMEMDS through fs.
	STOP(fs,0xB8000,77,24,'A,0xC)	; 'A' for Allmem
	STOP(fs,0xB8000,78,24,'l,0xC)	; 'l' for Allmem
	STOP(fs,0xB8000,79,24,'l,0xC)	; 'l' for Allmem

	mov ax,#0x0003
	int #0x10

; Print some stuff

	seg fs
	mov eax,[IRQ_INT_BASE*4]
	seg fs
	mov ebx,[IRQ_INT_BASE*4+8*4]

	rz__z
	STOP(fs,0xB8000,71,9,'a,0xB)	; another 'a'




; Restore Interrupt Controller
	STOP(es,0,58,9,'R,0x61)
	;call RM_Setup_RM_8259
	STOP(es,0,59,9,'8,0x61)
	;call RM_Flush_Interrupts
	STOP(es,0,70,9,'B,0xB)		; 'B' for Back

; Reenable Interruptions
	call RM_Unmask_IRQs
	call RM_Enable_NMI
	ppmov(ds,cs)
	STOP(es,0,71,9,'a,0xB)		; 'A' for Back


	__({"Back in real mode:"},CRLF)
	STOP(es,0,72,9,'c,0xB)		; 'c' for Back
	z_z

; That's all, folks
	STOP(es,0,73,9,'k,0xB)		; 'k' for Back
	CGETC({CRLF,"Hit any key to reboot."})
	STOP(es,0,74,8,'!,0xB)		; '!' for Back!
; Reboot !
	mov ax,#0x1234
	ppmov(es,#0)
	seg es
	mov [0x472],#0x1234
	seg es
	mov [0x467],#0x1234
	mov al,#0xFE	; Pulse "reset" = 8042 pin 0
	out #0x64,al
	___die___	; It takes some time to reboot, so let's wait a bit.

})


})dnl --- end of head.m4
dnl
