#!/bin/bash

############################## CONFIGURATION #################################

case "$VERSION" in
  zsh*) ;;
  *)
   setopt () { : } ;;
esac

default-value () {
  eval A="$"$1
  if [ -z "$A" ] ; then
    eval export "$1=$2"
    eval : "$1 = $""$1"
  else
    eval export "$1"
  fi
  : TUNES=$TUNES
  : BOOTREC=$BOOTREC LOADER=$LOADER API=$API MODULES=$MODULES
  : LOADMODE=$LOADMODE BLOCKSIZE=$BLOCKSIZE
  : DEST=$DEST INSTTOOL=$INSTTOOL
}

config () {
  default-value TUNES ../../..
  default-value DEST "${TUNES}/bin/i386/TUNES.b"
  default-value ARCH "$(arch)"
  default-value INSTTOOL $TUNES/bin/$ARCH/tunesinsttool
  default-value FLOPPY /dev/fd0
  default-value LOADLINKERN = /dos/c/tools/tunes.b
  default-value LILOKERN /boot/TUNES

  default-value BOOTREC bootrec.b
  default-value LOADER loader.b
  default-value API biosapi.b
  default-value MODULES

  default-value LOADMODE C
  default-value BLOCKSIZE 512

  default-value OPTION_SIG "%%InStTiMeOpTiOn%%"
  default-value OPTION_NUM 1

  : TUNES=$TUNES
  : BOOTREC=$BOOTREC LOADER=$LOADER API=$API MODULES=$MODULES
  : LOADMODE=$LOADMODE BLOCKSIZE=$BLOCKSIZE
  : DEST=$DEST INSTTOOL=$INSTTOOL


  return 0		# The rest is now done from the Makefile
}



############################### IO ROUTINES ##################################

TMP=/tmp/tmp-tunes.install.sh--$HOST-$$-


DD ()
{
  RES="${1}" ; shift
  setopt +x
  dd bs=${BLOCKSIZE} conv=sync "$@" 2>${TMP}
  setopt -x
  read a b < ${TMP}
  : a=$a b=$b
  if [ -n "${RES}" ] ; then
    eval "${RES}="'$['"${a}""]"
  fi
}

rmtmp ()
{
  /bin/rm -f -- ${TMP}* "$@"
}

abort ()
{
  b=$?
  if [ -f "$DEST" ] ; then rmtmp "$DEST" ; else rmtmp ; fi
  if [ -n "$1" ] ; then
    a=$1
    shift
    echo "$@"
    exit $a
  else
    exit $b
  fi
}


##############################################################################
# Adjust the parameters in the bootable image $DEST from the arguments
make_params ()
{

  ### Tell the LOADER what LOADMODE to use ###
  if [ $LOADMODE = L ] ; then
    echo 'LOADMODE "L" is autodetected.' \
	 'Defaulting to mode "C" on a 1.4M floppy'
    LOADMODE=C
  fi

  for i in BOOTRECSIZE LOADERSIZE SYSSIZE # LOADMODE
     do echo -n $i= ; eval echo '$'$i ; done

  export BOOTRECSIZE LOADERSIZE SYSSIZE LOADMODE
  ${INSTTOOL} adjust_i386_image $DEST

}


make () {
  trap abort EXIT TERM HUP
  case "${LOADMODE}" in
    C|L)
		mkdir $(dirname $DEST) 2>/dev/null
		{
		 DD BOOTRECSIZE < $BOOTREC
	         dd if=$LOADER ibs=256 skip=1 2>/dev/null > ${TMP}xxx
		 DD LOADERSIZE < ${TMP}xxx
		 # adjust size for LILO loading: at least 2KB for loader !!!
		 if [ $LOADERSIZE -lt 4 ] ; then
		   dd if=/dev/zero bs=512 count=$[4-$LOADERSIZE] 2>/dev/null
		   LOADERSIZE=4
		 fi
		 { dd if=$API ibs=4096 skip=1
		   if [ -n "$MODULES" ] ; then cat $MODULES ; fi
		 } > ${TMP}xxx
                 DD SYSSIZE < ${TMP}xxx
		} > ${DEST}
		make_params
		;;
    D)		echo "Load mode Not supported yet"
		;;
    H)		echo "Load mode Not supported yet"
		;;
    *)		echo "Never heard about this LOADMODE $LOADMODE"
		;;
  esac
  DEST=
}


altdest () {
  case "x$1" in
    x) ;;
    x/dev/*fd*)
	( cat "$DEST"
	  setopt +x
	  dd bs=3k count=1 if=/dev/zero 2>/dev/null
	  setopt -x
	) > "$1"
	echo "" # Beep
	#mcopy $DEST a:
	;;
    x*) cat "$DEST" > "$1" ;;
  esac
}

option () {
  if [ -z "$OPTION_SIG0" ] ; then
    OPTION_SIG0="$OPTION_SIG"
  fi
  export OPTION_SIG="${OPTION_SIG0}$1" OPTION_NUM="$2"
  ${INSTTOOL} toggle_i386_option ${API} || { echo Error $? ; exit }
  shift 2
  if [ -n "$*" ] ; then option $@ ; fi
  touch $API
}

options () {
  cat ${@:-options} | while read a b ; do option $a $b ; done
}

#############################################################################
config	# read the configuration
MODULES=${MODULES:-/dev/null}


if [ -z "$*" ]
then
  make
else
  $*
fi
